/*
 *	Time Lock Sample
 *	Copyright (c) 1994 Robust Software
 *
 *	INSTALL module
 *	This is a very simplistic program which demonstrates the use of the
 *	Software Copy Protection's Time Locks.
 *
 */

#include "windows.h"

#include <ctype.h>
#include <direct.h>

#include "ddeclnt.h"
#include "install.h"
#include <stdlib.h>
#include "codelock.h"
#include "resource.h"

/************************************************************************\
 |                      INCLUDE FILE DEFINITIONS                        |
\************************************************************************/
// Description of the program to be installed--for window titles, etc.
// NOTE: be sure to put this name in INSTALL.RC also!
#define PROGRAM_NAME			"Time Lock Sample"

// Main program name (to be executed by program's icon in Program Manager)
#define MAIN_PROGRAM			"TIMELOCK.EXE"

// Default directory (under Windows drive's root) to install program to
#define DEF_INSTALL_PATH		"TIMELOCK"

// Program Manager group file name
#define PROGMAN_GROUP			"CopyProt Samples"

#define PATH_SIZE				144
#define COPY_BUFSIZE			4096
#define SHADE_INCR				5
#define INSTALL_BACKGD			RGB(0, 0, 128)	// Background color
#define GRAY_COLOR				RGB(192, 192, 192)

// File copy status window size
#define COPY_WIDTH				400
#define COPY_HEIGHT				140

// "% complete" bar coordinates within status window
#define BAR_X					50
#define BAR_Y					80
#define BAR_WIDTH				300
#define BAR_HEIGHT				40

#define FILES_Y					(10 + charHeight)
#define FILES_HEIGHT			20

/************************************************************************\
 |						PROGRAM GLOBAL VARIABLES						|
\************************************************************************/
HWND hMainWnd;
HANDLE hInst;
char szWindowsPath[PATH_SIZE];
char szSourcePath[PATH_SIZE];
char szModulePath[PATH_SIZE];
char szInstallPath[PATH_SIZE];
int nDefaultDrive;
WORD wScreenWidth;
WORD wScreenHeight;
LONG lTotalFileSize;
LONG lBytesCopied;
int nFilesToCopy;
int nFilesCopied;

LPSTR lpInstallFiles[] = {		// Files to copy from diskette
	"TIMELOCK.EXE",
	"CODELOCK.DLL",
	NULL
};

BYTE copyBuf[COPY_BUFSIZE];
FARPROC lpfnOldDialogFilter;
BOOL bTrapInsAndDel;
int charWidth;
int charHeight;
char gszAuthNum[255];
char gszRegNum[255];
char gszProductID[] = "timelock";
char gszAuthNum[255];
char gszRegNum[255];

/************************************************************************\
 |							FUNCTION PROTOTYPES							|
\************************************************************************/
int PASCAL WinMain(HANDLE hInstance, HANDLE hPrevInstance,
	LPSTR lpCmdLine, int nCmdShow);
BOOL InitApplication(HANDLE hInstance);
BOOL InitInstance(HANDLE hInstance, int nCmdShow);
long FAR PASCAL _export MainWndProc(HWND hWnd, unsigned message,
	WORD wParam, LONG lParam);
VOID PaintWindow(HWND hWnd, HDC hDC);
DWORD OurGetTextExtent(HDC hDC, LPSTR str);
VOID StringOut(HDC hDC, int x, int y, LPSTR str);
VOID PaddedStringOut(HDC hDC, int x, int y, LPSTR str, int nFieldWidth,
	COLORREF padColor);
BOOL FAR PASCAL _export InstallDialogProc(HWND hDlg, unsigned message,
	WORD wParam, LONG lParam);
VOID DoTheInstall(HWND hWnd);
BOOL DestFileExists(LPSTR lpFile);
BOOL CreatePath(PSTR pPath);  
DWORD GetFileSize(LPSTR lpFile);
BOOL CopyFiles(HWND hWnd);
BOOL CopyFile(LPSTR lpFile, HWND hWndStatus);
long FAR PASCAL _export CopyFileProc(HWND hWnd, unsigned message,
	WORD wParam, LONG lParam);
VOID UpdateCopyWindow(HWND hCopyFileWnd);
BOOL AddToDesktop(HWND hWnd);
VOID MakeItAPath(LPSTR path);
VOID FixUpPath(LPSTR path);
int OurDialogBox(BOOL bTrapFlag, HINSTANCE hInst, LPSTR lpTemplate,
	HWND hWndOwner, DLGPROC lpfnDialog);
DWORD FAR PASCAL _export DialogFilter(int nCode, WORD wParam,
	DWORD lParam);
BOOL FAR PASCAL AuthDialogProc(HWND hDlg, unsigned message,
  WORD wParam, LONG lParam);

//
//	WinMain
//

int PASCAL WinMain(HANDLE hInstance, HANDLE hPrevInstance,
	LPSTR lpCmdLine, int nCmdShow)
{
	hInst = hInstance;

	// Other instances of app running?
	if (!hPrevInstance)
		// Initialize shared things
		if (!InitApplication(hInstance))
			// Exits if unable to initialize
			return FALSE;

	// Perform initializations that apply to a specific instance 
	if (!InitInstance(hInstance, nCmdShow))
		return FALSE;

	DoTheInstall(hMainWnd);

	// Install a 10 day time lock for our product.
	if (InstallTimeLock(gszProductID, 10) == CODELOCK_FAILURE) {
		MessageBox(hMainWnd, "InstallTimeLock failed.",
			"Time Lock Sample Setup", MB_OK);
		return 1;
	}

    // All done, release system resources
	DestroyWindow(hMainWnd);

	return 0;
}

//
//	InitApplication
//

BOOL InitApplication(HANDLE hInstance)
{
	WNDCLASS wc;

	// Main window class structure
	wc.style = NULL;
	wc.lpfnWndProc = (WNDPROC)MainWndProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;

	// Set icon for fast Alt+Tab switching
	wc.hIcon = LoadIcon(hInstance, "InstallMin");
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = GetStockObject(NULL_BRUSH);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = "InstallWClass";

	if (!RegisterClass(&wc))
		return FALSE;

	// Copy-file status window class structure
	wc.style = NULL;
	wc.lpfnWndProc = (WNDPROC)CopyFileProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hIcon = NULL;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);

    // Windows automatically deletes the brush created below
	wc.hbrBackground = CreateSolidBrush(GRAY_COLOR);

	wc.lpszMenuName = NULL;
	wc.lpszClassName = "FileCopyClass";

	if (!RegisterClass(&wc))
		return FALSE;

	return TRUE;
}

//
//	InitInstance
//

BOOL InitInstance(HANDLE hInstance, int nCmdShow)
{
	HDC hDC;
	TEXTMETRIC tmSystem;
	char szWinTitle[80];

	// Get text metrics for system font
	hDC = CreateIC("DISPLAY", NULL, NULL, NULL);
	GetTextMetrics(hDC, &tmSystem);
	DeleteDC(hDC);

	charHeight = tmSystem.tmHeight + tmSystem.tmExternalLeading;
	charWidth = tmSystem.tmAveCharWidth;

	wScreenWidth = GetSystemMetrics(SM_CXSCREEN);
	wScreenHeight = GetSystemMetrics(SM_CYSCREEN);

	wsprintf(szWinTitle, "%s Setup", PROGRAM_NAME);
	hMainWnd = CreateWindow("InstallWClass", "Time Lock Sample Setup",				// Fill in program name here
		WS_POPUP | WS_VISIBLE, 0, 0, wScreenWidth, wScreenHeight,
		NULL, NULL, hInstance, NULL);

	if (!hMainWnd)
		return FALSE;

	ShowWindow(hMainWnd, nCmdShow);
	UpdateWindow(hMainWnd);

	return TRUE;
}

//
//	MainWndProc
//

long FAR PASCAL _export MainWndProc(HWND hWnd, unsigned message,
	WORD wParam, LONG lParam)
{
	HDC hDC;
	PAINTSTRUCT ps;

	switch (message) {
	case WM_PAINT:
		hDC = BeginPaint(hWnd, &ps);
    	PaintWindow(hWnd, hDC);
    	EndPaint(hWnd, &ps);
		break;

	case WM_CLOSE:
		DestroyWindow(hWnd);
		break;

	case WM_DESTROY:				// Window being destroyed
		PostQuitMessage(0);
		break;

	case WM_COMMAND:
		switch(wParam) {
		case IDM_EXIT:
			DestroyWindow(hWnd);
			break;
		}
        break;

	default:						// Passes it on if unprocessed
		return(DefWindowProc(hWnd, message, wParam, lParam));
	}

	return NULL;
}

//
//	PaintWindow
//
//	Services our window's WM_PAINT message.
//

VOID PaintWindow(HWND hWnd, HDC hDC)
{
	HBRUSH hBlueBrush;
	RECT rcClient, rcFill;
	HFONT hBigHelv, hSmallHelv, hOldFont;
	int y, wWidth, wHeight;
	DWORD dwBrite;

	// Draw cool-looking black-to-blue shading
	GetClientRect(hWnd, &rcClient);
	rcFill = rcClient;
    wHeight = rcClient.bottom - rcClient.top;
	for (y = rcClient.top; y < rcClient.bottom; y += SHADE_INCR) {
		rcFill.top = y;
		rcFill.bottom = y + SHADE_INCR;
		dwBrite = 255L * (long)(y - rcClient.top) * (long)(y - rcClient.top);
        dwBrite = dwBrite / wHeight / wHeight; 
		hBlueBrush = CreateSolidBrush(RGB(0, 0, dwBrite));
		FillRect(hDC, &rcFill, hBlueBrush);
        DeleteObject(hBlueBrush);
	}

	hBigHelv = CreateFont(45, 45, 0, 0, FW_HEAVY, 1, 0, 0,
		DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS,
		PROOF_QUALITY, VARIABLE_PITCH | FF_SWISS, "Helv");
	hSmallHelv = CreateFont(16, 16, 0, 0, FW_BOLD, 1, 0, 0,
		DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS,
		PROOF_QUALITY, VARIABLE_PITCH | FF_SWISS, "Helv");

	if (!hBigHelv || !hSmallHelv)
		return;

	hOldFont = SelectObject(hDC, hBigHelv);
    wWidth = LOWORD(OurGetTextExtent(hDC, PROGRAM_NAME));
	SetTextColor(hDC, RGB(255, 255, 255));
	SetBkMode(hDC, TRANSPARENT);

	StringOut(hDC, 25, 25, PROGRAM_NAME);
    StringOut(hDC, 80, 75, "Setup"); 

	SelectObject(hDC, hOldFont);
	DeleteObject(hBigHelv);
    DeleteObject(hSmallHelv);
}

//
//	AuthDialogProc
//

BOOL FAR PASCAL AuthDialogProc(HWND hDlg, unsigned message,
	WORD wParam, LONG lParam)
{
	switch (message) {
	case WM_INITDIALOG:
		SetDlgItemText(hDlg, IDC_REGNUM, (LPSTR)gszRegNum);
		return TRUE;

	case WM_CLOSE:
		EndDialog(hDlg, TRUE);	// Exits the dialog box
		return TRUE;

	case WM_COMMAND:
		switch (wParam) {
		case IDOK:
			GetDlgItemText(hDlg, IDC_AUTHNUM, (LPSTR)gszAuthNum, sizeof(gszAuthNum));
			EndDialog(hDlg, TRUE);
			return TRUE;

		case IDCANCEL:
			EndDialog(hDlg, FALSE);
			return TRUE;
		}
		break;
	}

	return FALSE;
}

//
//	OurGetTextExtent
//

DWORD OurGetTextExtent(HDC hDC, LPSTR str)
{
	int len = lstrlen(str);

	return GetTextExtent(hDC, str, len);
}

//
//	StringOut
//

VOID StringOut(HDC hDC, int x, int y, LPSTR str)
{
	TextOut(hDC, x, y, str, lstrlen(str));
}

//
//	PaddedStringOut
//

VOID PaddedStringOut(HDC hDC, int x, int y, LPSTR str, int nFieldWidth,
	COLORREF padColor)
{
	RECT r;
	DWORD dwTextExtent;
	int nStrLen;
	HBRUSH hBrush;

	nStrLen = lstrlen(str);
	dwTextExtent = GetTextExtent(hDC, str, nStrLen);
	r.left = x + LOWORD(dwTextExtent);
	r.top = y;
	r.right = x + nFieldWidth;
	r.bottom = y + HIWORD(dwTextExtent);

	SetBkColor(hDC, padColor);
	hBrush = CreateSolidBrush(padColor);
	TextOut(hDC, x, y, str, lstrlen(str));
	FillRect(hDC, &r, hBrush);
	DeleteObject(hBrush);
}

//
//	CenterText
//

VOID CenterText(HDC hDC, int leftX, int topY, int wWidth, LPSTR str,
	WORD wOpaque)
{
	DWORD dwTemp;
	RECT rect;
	WORD wTextWidth, wTextHeight;
    int len;

    len = lstrlen(str);
	dwTemp = GetTextExtent(hDC, str, len);
	wTextWidth = LOWORD(dwTemp);
	wTextHeight = HIWORD(dwTemp);
	SetRect(&rect, leftX, topY, leftX + wWidth, topY + wTextHeight + 1);
	ExtTextOut(hDC, leftX + (wWidth - wTextWidth)/2, topY, wOpaque,
		&rect, str, len, NULL);
}

//
//	InstallDialogProc
//

BOOL FAR PASCAL _export InstallDialogProc(HWND hDlg, unsigned message,
	WORD wParam, LONG lParam)
{
	switch (message) {
	case WM_INITDIALOG:
		wsprintf(szInstallPath, "%c:\\%s", szWindowsPath[0],
			(LPSTR)DEF_INSTALL_PATH);
		SendDlgItemMessage(hDlg, IDD_INSTALL_DIRECTORY, WM_SETTEXT,
			0, (DWORD)(LPSTR)szInstallPath);
		return TRUE;

	case WM_CLOSE:
		SendDlgItemMessage(hDlg, IDD_INSTALL_DIRECTORY, WM_GETTEXT,
			PATH_SIZE, (DWORD)(LPSTR)szInstallPath);
		EndDialog(hDlg, TRUE);	// Exits the dialog box
		return TRUE;

	case WM_COMMAND:
		switch(wParam) {
		case IDOK:
			SendDlgItemMessage(hDlg, IDD_INSTALL_DIRECTORY,
				WM_GETTEXT, PATH_SIZE, (DWORD)(LPSTR)szInstallPath);
			EndDialog(hDlg, TRUE);
			return TRUE;

		case IDCANCEL:
			EndDialog(hDlg, FALSE);
			return TRUE;
		}

		break;
	}

	return FALSE;
}

//
//	DoTheInstall
//

VOID DoTheInstall(HWND hWnd)
{
	int retVal, len;
	char tempPath[PATH_SIZE], tempBuf[256], tempBuf2[80];
	BOOL bReInstalling;
	char szTmpBuf[255];
	WORD protCode;
	int x;

	// Get full name of this program as it was executed
	GetModuleFileName(hInst, szModulePath, PATH_SIZE);

    // Remove filename from szModulePath to leave source directory name
    MakeItAPath(szModulePath);

	FixUpPath(szModulePath);

	// Install source path is the same as our module's path
	lstrcpy(szSourcePath, szModulePath);

    // Get Windows directory (user's private directory on networks)
	GetWindowsDirectory(szWindowsPath, PATH_SIZE);
	nDefaultDrive = szWindowsPath[0];
    FixUpPath(szWindowsPath);

    bReInstalling = FALSE;

	// Indicate that we're busy to our user
	SetCursor(LoadCursor(NULL, IDC_WAIT));
	
	// Let's get a unique random registration ID which is atleast 5
	// characters long.
	GenerateRegistrationID(gszRegNum, sizeof(gszRegNum), 5);
	
	// Our authorization code is our protection code which is generated
	// from our registration number.  The RegIDToProtCode function generates
	// a WORD value which corresponds to the string passed into it.
	protCode = RegIDToProtCode(gszRegNum);

	// Display a message box indicating the authorization number the user
	// should enter.  This is only done to make this application a demo.
	wsprintf((LPSTR)szTmpBuf, "The authorization number is %u.  The authorization number would not normally be shown to the user through a dialog box such as this one;  this was done here because this is a demo application.",
		protCode);
	MessageBox(NULL, (LPSTR)szTmpBuf, "Time Lock Sample Setup", MB_OK);	
	
	// Not busy any more.
	SetCursor(LoadCursor(NULL, IDC_ARROW));
	
	// Show the user a dialog box requesting an authorization ID number.
	retVal = OurDialogBox(FALSE, hInst, "AuthDialog", hWnd, AuthDialogProc);
	if (!retVal) {
		MessageBox(hWnd, "Setup has been aborted.",
			"Time Lock Sample Setup", MB_OK);
	}

	// gszAuthNum is the authorization number which the user entered into
	// the dialog box entry field.  Lets get the protection code for this
	// number and compare it against the one we generated for our gszRegNum.
	// If their the same, then the installation will continue, if not, a
	// message box is displayed to the user indicating that the wrong
	// authorization number was entered.
	protCode = RegIDToProtCode(gszAuthNum);
	wsprintf((LPSTR)szTmpBuf, "%u", RegIDToProtCode(gszRegNum));
	for (x = 0; x < sizeof(szTmpBuf); x++)
		if (gszAuthNum[x]!=szTmpBuf[x] || !szTmpBuf[x])
			break;
	if (x < lstrlen((LPSTR)szTmpBuf)) {
		MessageBox( hWnd, "The authorization number is invalid.  Time Lock Sample will not be installed.",
		  "Time Lock Sample Setup", MB_OK );
		return;
	}
	
	for (;;) {
		retVal = OurDialogBox(FALSE, hInst, "InstallDialog", hWnd,
			InstallDialogProc);
		FixUpPath(szInstallPath);

		if (!retVal) {
			MessageBox(hWnd, "Setup has been aborted.",
				"ProgLock Sample Setup", MB_OK);
			return;
        }

		if (DestFileExists(lpInstallFiles[0])) {
        	// Remove trailing backslash from path
			lstrcpy(tempPath, szInstallPath);
			if ((len = lstrlen(tempPath)) > 0)
            	tempPath[len-1] = '\0';

			wsprintf(tempBuf,
				"%s is already installed in %s.  Do you wish to re-install it?",
            	(LPSTR)PROGRAM_NAME, (LPSTR)tempPath);
            wsprintf(tempBuf2, "%s Setup", (LPSTR)PROGRAM_NAME);
			retVal = MessageBox(hWnd, tempBuf, tempBuf2,
				MB_YESNOCANCEL);
			if (retVal == IDCANCEL)
				return;
			else if (retVal == IDNO)
				continue;
			else
            	bReInstalling = TRUE;	// Can use if needed
		}

		// We have our install path...create it if necessary
		if (!CreatePath(szInstallPath)) {
			wsprintf(tempBuf, "Cannot create directory: %s",
				(LPSTR)szInstallPath);
			MessageBox(hWnd, tempBuf, NULL, MB_OK | MB_ICONSTOP);
			continue;
		}

		break;
    }

	// Now copy the files to the destination
	if (!CopyFiles(hWnd))
    	return;

	if (!AddToDesktop(hWnd))
		return;

	// Create any necessary data files here

	wsprintf(tempBuf, "%s Setup is complete!", (LPSTR)PROGRAM_NAME);
	wsprintf(tempBuf2, "%s Setup", (LPSTR)PROGRAM_NAME);
	MessageBox(hWnd, tempBuf, tempBuf2, MB_OK | MB_ICONINFORMATION);
	return;
}

//
//	DestFileExists
//

BOOL DestFileExists(LPSTR lpFile)
{
	char fileName[128];
	HFILE hFile;
    OFSTRUCT of;

	wsprintf(fileName, "%s%s", (LPSTR)szInstallPath, (LPSTR)lpFile);

    // See if the file exists (does not return open file handle)
	hFile = OpenFile(fileName, &of, OF_EXIST);
	if (hFile == -1)
		return FALSE;
	else
		return TRUE;
}

//
//	CreatePath
//
//	Creates the given full path (or the parts of it that don't already exist).
//  Returns TRUE if the path was created OK (or it already existed), FALSE
//	if an error occured.
//

BOOL CreatePath(PSTR pPath)
{
	int nDrive;						// 1=A:, 2=B:, etc.
	char tempBuf[PATH_SIZE], partialPath[PATH_SIZE];
	PSTR p, q, pNext;
    int len;

	lstrcpy(tempBuf, pPath);

	// Use AnsiUpper() for international support
	nDrive = toupper(tempBuf[0]) - 'A' + 1;

    // Make sure drive letter is valid; then change to that drive
	if (nDrive < 1 || nDrive > 26 || tempBuf[1] != ':' ||
    	tempBuf[2] != '\\' || _chdrive(nDrive))
		return FALSE; 

    // Remove trailing backslash, if any
	len = lstrlen(tempBuf);
	if (len && tempBuf[len - 1] == '\\')
    	tempBuf[--len] = '\0';

	if (!len || tempBuf[len - 1] == ':')
		return TRUE;

	for (pNext = tempBuf; *pNext && *pNext != '\\'; ++pNext)
		;

	if (!*pNext)
		return FALSE;					// We gotta have a backslash

	++pNext;
	if (!*pNext)
    	return TRUE;					// Can't create root directory

    // Create intermediate paths, one at a time as necessary
	for (;;) {
		while (*pNext && *pNext != '\\')
			++pNext;

        // Copy tempBuf through (but not including) pNext to partialPath
		for (p = tempBuf, q = partialPath; p < pNext;)
			*q++ = *p++;

		*q = '\0';

        // Change to the path; if that fails, create it--if that fails, ERR!
		if (chdir(partialPath) && mkdir(partialPath))
			return FALSE;

		if (*pNext == '\\')
			++pNext;
		else
        	break;
	}

	return TRUE;
}

//
//	GetFileSize
//

DWORD GetFileSize(LPSTR lpFile)
{
    DWORD lSize;
	HFILE hFile;
	OFSTRUCT of;

	if ((hFile = OpenFile(lpFile, &of, OF_READ)) == -1)
		return 0L;

	lSize = _llseek(hFile, 0L, SEEK_END);
	_lclose(hFile);
	return(lSize);
}

//
//	CopyFiles
//

BOOL CopyFiles(HWND hWnd)
{
	DWORD lThisFileSize;
	int i;
    HWND hFileCopyWnd;
	char szFile[PATH_SIZE], textBuf[PATH_SIZE+20];
	HPEN hBlackPen, hOldPen;
	HDC hDC;
	HBRUSH hOldBrush;

	for (i=0; lpInstallFiles[i]; ++i) {
		wsprintf(szFile, "%s%s", (LPSTR)szSourcePath,
			(LPSTR)lpInstallFiles[i]);
		if ((lThisFileSize = GetFileSize(szFile)) == -1L) {
			wsprintf(textBuf, "Cannot read file: <%s>",
            	(LPSTR)szFile);
        	MessageBox(hWnd, textBuf, NULL, MB_OK | MB_ICONSTOP);
			return FALSE;
        }

		lTotalFileSize += lThisFileSize;
	}

	nFilesToCopy = i;

    // Create file copy status window
	hFileCopyWnd = CreateWindow("FileCopyClass", "", WS_POPUP | WS_VISIBLE |
		WS_BORDER, (wScreenWidth - COPY_WIDTH) / 2, (wScreenHeight - COPY_HEIGHT) / 2,
		COPY_WIDTH, COPY_HEIGHT, hWnd, NULL, hInst, NULL);

	if (!hFileCopyWnd) {
		MessageBox(hWnd, "Error starting file-copy window", NULL,
        	MB_OK | MB_ICONSTOP);
		return FALSE;
    }

	ShowWindow(hFileCopyWnd, SW_SHOW);	// Show the window
	UpdateWindow(hFileCopyWnd);

	hDC = GetDC(hFileCopyWnd);
	hBlackPen = GetStockObject(BLACK_PEN);

	hOldPen = SelectObject(hDC, hBlackPen);
	hOldBrush = SelectObject(hDC, GetStockObject(WHITE_BRUSH));
	Rectangle(hDC, BAR_X, BAR_Y, BAR_X+BAR_WIDTH+1, BAR_Y+BAR_HEIGHT+1);

	SetBkColor(hDC, GRAY_COLOR);
	StringOut(hDC, BAR_X, BAR_Y - charHeight - 5, "0%");
	StringOut(hDC, BAR_X + BAR_WIDTH - 2*charWidth,
		BAR_Y - charHeight - 5, "100%");

	wsprintf(textBuf, "Now Copying %s Files", (LPSTR)PROGRAM_NAME);
	CenterText(hDC, BAR_X, 10, BAR_WIDTH, textBuf, ETO_OPAQUE);

	SelectObject(hDC, hOldPen);
	SelectObject(hDC, hOldBrush);
	ReleaseDC(hFileCopyWnd, hDC);

    // Redraw main window
	UpdateWindow(hWnd);

	for (nFilesCopied=0; nFilesCopied < nFilesToCopy; ++nFilesCopied) {
    	CopyFile(lpInstallFiles[nFilesCopied], hFileCopyWnd); 
		InvalidateRect(hFileCopyWnd, NULL, FALSE);
		UpdateCopyWindow(hFileCopyWnd);
	}

	DestroyWindow(hFileCopyWnd);
    UpdateWindow(hWnd);

    return TRUE;

}

//
//	CopyFile
//

BOOL CopyFile(LPSTR lpFile, HWND hWndStatus)
{
	char szInFile[PATH_SIZE], szOutFile[PATH_SIZE], textBuf[PATH_SIZE+20];
	HFILE hInFile, hOutFile;
	OFSTRUCT ofIn, ofOut;
	int nRead, nWritten;

	wsprintf(szInFile, "%s%s", (LPSTR)szSourcePath, (LPSTR)lpFile);
	wsprintf(szOutFile, "%s%s", (LPSTR)szInstallPath, (LPSTR)lpFile);

	hInFile = OpenFile(szInFile, &ofIn, OF_READ);
	if (hInFile == -1) {
    	wsprintf(textBuf, "Error opening file <%s>", (LPSTR)szInFile);
    	MessageBox(NULL, textBuf, NULL, MB_OK | MB_ICONSTOP);
		return FALSE;
    }

	hOutFile = OpenFile(szOutFile, &ofOut, OF_CREATE | OF_WRITE);
	if (hOutFile == -1) {
		_lclose(hInFile);
		wsprintf(textBuf, "Error opening file <%s>", (LPSTR)szOutFile);
    	MessageBox(NULL, textBuf, NULL, MB_OK | MB_ICONSTOP);
		return FALSE;
	}

	while ((nRead = _lread(hInFile, copyBuf, COPY_BUFSIZE)) > 0) {
		if ((nWritten = _lwrite(hOutFile, copyBuf, nRead)) != nRead) {
			_lclose(hInFile);
			_lclose(hOutFile);
			wsprintf(textBuf, "Error writing file <%s> or disk full",
			  (LPSTR)szOutFile);
			MessageBox(NULL, textBuf, NULL, MB_OK | MB_ICONSTOP);
			return FALSE;
		}

		lBytesCopied += nWritten;
		UpdateCopyWindow(hWndStatus);
	}

	_lclose(hInFile);
	_lclose(hOutFile);
    return TRUE;
}

//
//	CopyFileProc
//

long FAR PASCAL _export CopyFileProc(HWND hWnd, unsigned message,
	WORD wParam, LONG lParam)
{
	HDC			hDC;
    PAINTSTRUCT	ps;

	switch(message) {
	case WM_PAINT:
		hDC = BeginPaint(hWnd, &ps);
		EndPaint(hWnd, &ps);
		break;

	default:
		return(DefWindowProc(hWnd, message, wParam, lParam));
	}

    return NULL;
}

//
//	UpdateCopyWindow
//

VOID UpdateCopyWindow(HWND hCopyFileWnd)
{
	HDC hDC;
	HBRUSH hBlueBrush, hOldBrush;
	DWORD dwTemp;
    RECT rect;

	hDC = GetDC(hCopyFileWnd);
	hBlueBrush = CreateSolidBrush(RGB(0, 0, 128));
	hOldBrush = SelectObject(hDC, GetStockObject(NULL_BRUSH));

    // An ounce of prevention...
	if (lBytesCopied > lTotalFileSize || lTotalFileSize == 0L) 
		dwTemp = BAR_WIDTH;
	else {
		dwTemp = lBytesCopied * BAR_WIDTH;
		dwTemp /= lTotalFileSize;
	}

	SetRect(&rect, BAR_X, BAR_Y, BAR_X + dwTemp, BAR_Y + BAR_HEIGHT);
	FillRect(hDC, &rect, hBlueBrush);

	SetBkColor(hDC, GRAY_COLOR);

	SelectObject(hDC, hOldBrush);
	DeleteObject(hBlueBrush);

	ReleaseDC(hCopyFileWnd, hDC);
}

//
//	AddToDesktop
//

BOOL AddToDesktop(HWND hWndParent)
{
	HWND hDDE, hServerWnd;				// PROGMAN's DDE server window
    char DDECommand[128], textBuf[128];

	hDDE = DDEClientOpen(hWndParent, "PROGMAN", "PROGMAN");
	if (hDDE == NULL)
		goto weGotAnError;

	hServerWnd = (HWND)GetWindowWord(hDDE, WW_SERVER);

	BringWindowToTop(hServerWnd);
	UpdateWindow(hServerWnd);

	wsprintf(DDECommand, "[CreateGroup(%s)]", (LPSTR)PROGMAN_GROUP);
	if (!DDEExecute(hDDE, DDECommand, 3))
		goto weGotAnError;

	wsprintf(DDECommand, "[ShowGroup(%s,1)]", (LPSTR)PROGMAN_GROUP);
	if (!DDEExecute(hDDE, DDECommand, 3))
    	goto weGotAnError;

	wsprintf(DDECommand, "[AddItem(%s%s,%s,,,,,%s,,)]",
		(LPSTR)szInstallPath, (LPSTR)MAIN_PROGRAM,
		(LPSTR)PROGMAN_GROUP, (LPSTR)szInstallPath);

	if (!DDEExecute(hDDE, DDECommand, 3))
		goto weGotAnError;

	DDEClientClose(hDDE);
	BringWindowToTop(hWndParent);
    UpdateWindow(hWndParent);
	return TRUE;

weGotAnError:
	wsprintf(textBuf, "Cannot add %s to desktop", (LPSTR)PROGRAM_NAME);
	MessageBox(hWndParent, textBuf, NULL,
		MB_OK | MB_ICONEXCLAMATION);
	if (hDDE != NULL)
		DDEClientClose(hDDE);

	BringWindowToTop(hWndParent);
    UpdateWindow(hWndParent);
	return FALSE;
}

//
//	OurDialogBox
//
//	Called in place of DialogBox(); it installs our dialog box message
//	filter before calling DialogBox().  The message filter catches
//	F1 keystrokes & xlates to HELP messages to provide context-sensitive
//	help in dialog boxes.
//

int OurDialogBox(BOOL bTrapFlag, HINSTANCE hInst, LPSTR lpTemplate,
	HWND hWndOwner, DLGPROC lpfnDialog)
{
	int retVal;
    BOOL bOldTrapFlag;
	FARPROC lpfnDialogFilter, lpfnOldOldDialogFilter;

	// Save off current globals in local, recursively-stored variables
	lpfnOldOldDialogFilter = lpfnOldDialogFilter;
    bOldTrapFlag = bTrapInsAndDel;

	bTrapInsAndDel = bTrapFlag;

	// Install our message filter so we can process accelerators
    // (such as F1 for Help) in the dialog box
    lpfnDialogFilter = MakeProcInstance((FARPROC)DialogFilter, hInst);
	lpfnOldDialogFilter =  SetWindowsHook(WH_MSGFILTER, lpfnDialogFilter);

	retVal = DialogBox(hInst, lpTemplate, hWndOwner, lpfnDialog);

	UnhookWindowsHook(WH_MSGFILTER, lpfnDialogFilter);
	FreeProcInstance(lpfnDialogFilter);

	// Restore prior values of global vars
	lpfnOldDialogFilter = lpfnOldOldDialogFilter;
    bTrapInsAndDel = bOldTrapFlag;

	return retVal;
}

//
//	DialogFilter
//

DWORD FAR PASCAL _export DialogFilter(int nCode, WORD wParam, DWORD lParam)
{
	LPMSG lpMsg;
    WORD wSignalMsg;

	if (nCode < 0)
		return(DefHookProc(nCode, wParam, lParam, &lpfnOldDialogFilter));

	if (nCode == MSGF_DIALOGBOX) {
		lpMsg = (LPMSG)lParam;
		if (lpMsg->message == WM_KEYDOWN) {
			switch(lpMsg->wParam) {
			case VK_F1:
				wSignalMsg = IDC_DIALOG_HELP;
				break;

			case VK_INSERT:
				if (bTrapInsAndDel) {
					wSignalMsg = IDC_DIALOG_ADD;
					break;
				} else
					return 0L;

			case VK_DELETE:
				if (bTrapInsAndDel) {
					wSignalMsg = IDC_DIALOG_DELETE;
					break;
				} else
					return 0L;

			default:
				return 0L;		// Didn't handle the message
			}

			// lpMsg->hwnd is the handle of a dialog box control;
			// post a WM_COMMAND message to its parent, the dialog box
			PostMessage(GetParent(lpMsg->hwnd), WM_COMMAND, wSignalMsg, 0L);

			return 1L;				// We handled the message             
		}
	}

	return 0L;
}

//
//	MakeItAPath
//

VOID MakeItAPath(LPSTR path)
{
	LPSTR lpLastSlash, p;
	int len;

    lpLastSlash = NULL;
	for (p = path; *p; ++p) {
		if (*p == '\\')
			lpLastSlash = p;
		else if (*p == ':')
        	lpLastSlash = p + 1;
	}

	if (lpLastSlash != NULL)
		*lpLastSlash = '\0';

	if ((len = lstrlen(path)) > 0 && path[len-1] == ':') {
		path[len] = '\\';
		path[len+1] = '\0';
    }
}

//
//	FixUpPath
//
//	Insures that the given path has a drive spec, and that it ends in a
//	backslash.  It does not canonicalize the path or check it for validity.
//

VOID FixUpPath(LPSTR path)
{
	LPSTR p;
	char tempBuf[144];

	if (!isalpha(path[0]) || path[1] != ':' && isalpha(nDefaultDrive)) {
		tempBuf[0] = nDefaultDrive;
        tempBuf[1] = ':';
		tempBuf[2] = '\0';
	} else
		tempBuf[0] = '\0';

	lstrcat(tempBuf, path);

	if (tempBuf[0]) {
		for (p = tempBuf; *p; ++p)
			;

		if (*(p-1) != ':' && *(p-1) != '\\') {
			*p++ = '\\';
			*p = '\0';
		}
	}

    lstrcpy(path, tempBuf);
}

